/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package objets;

import java.io.IOException;

import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import types.Complexe;
import types.ErreurCalculException;

public class CSommeDansFormule extends CCalculBase {
	public CCalculBase calculASommer;
	public CCalculBase indiceDebut;
	public CCalculBase indiceFin;
	public CCalculBase pas;
	public String nomVariable;
	
	/**
	 * Constructeur vide pour serialization
	 */
	public CSommeDansFormule() {
	}
	
	public CSommeDansFormule(CListeObjets liste, CCalculBase calculASommerInit, CCalculBase indiceDebutInit,
			CCalculBase indiceFinInit, CCalculBase pasInit, String nomVariableInit) {
		super(liste);
		calculASommer = calculASommerInit;
		indiceDebut = indiceDebutInit;
		indiceFin = indiceFinInit;
		pas = pasInit;
		nomVariable = nomVariableInit;
	}

  @Override
  protected long nature() {
  	return natSommeDansFormule;
  }  

	@Override
	public boolean dependDeVariable(int ind) {
		return calculASommer.dependDeVariable(ind) || indiceDebut.dependDeVariable(ind)
			|| indiceFin.dependDeVariable(ind) || pas.dependDeVariable(ind);
	}

	@Override
	CCalculBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
		return new CSommeDansFormule(listeCible, calculASommer.getClone(listeSource, listeCible), indiceDebut.getClone(listeSource, listeCible),
				indiceFin.getClone(listeSource, listeCible), pas.getClone(listeSource, listeCible), nomVariable);
	}

	@Override
	public CCalculBase getCopie() {
		return new CSommeDansFormule(listeProprietaire, calculASommer.getCopie(), indiceDebut.getCopie(),
				indiceFin.getCopie(), pas.getCopie(), nomVariable);
	}
	
  // Ajout version 4.8
  @Override
  public void initialisePourDependance() {
  	super.initialisePourDependance();
  	calculASommer.initialisePourDependance();
  	indiceDebut.initialisePourDependance();
  	indiceFin.initialisePourDependance();
  	pas.initialisePourDependance();
  }

	// Modifi version 4.8
  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
    return memoriseDependance(super.dependDe(p) || calculASommer.dependDe(p) || indiceDebut.dependDe(p) || indiceFin.dependDe(p) || pas.dependDe(p));
  }

	// Modifi version 4.8
  @Override
  public boolean dependDePourReclassement(CElementBase p) {
  	if (elementTestePourDependDePourRec == p) return dependDeElementTestePourRec;
    return memoriseDependancePouRec(super.dependDePourReclassement(p) || calculASommer.dependDePourReclassement(p) || indiceDebut.dependDePourReclassement(p)
  	|| indiceFin.dependDePourReclassement(p) || pas.dependDePourReclassement(p));
  }
  
  @Override
  public boolean dependDePourBoucle(CElementBase p) {
  	return calculASommer.dependDePourBoucle(p) || indiceDebut.dependDePourBoucle(p)
  	|| indiceFin.dependDePourBoucle(p) || pas.dependDePourBoucle(p);
  }

  @Override
  public boolean existe(boolean infoRandom) {
  	return calculASommer.existe(infoRandom) && indiceDebut.existe(infoRandom)
  		&& indiceFin.existe(infoRandom) && pas.existe(infoRandom);
  }
  
  @Override
	public boolean estConstant() {
		return calculASommer.estConstant() && indiceDebut.estConstant() && indiceFin.estConstant() && pas.estConstant();
	}
  
  @Override
	public boolean estDefiniParObjetDans(CListeObjets listeOb) {
  	return
		calculASommer.estDefiniParObjetDans(listeOb)
		&& indiceDebut.estDefiniParObjetDans(listeOb)
  	&& indiceFin.estDefiniParObjetDans(listeOb)
  	&& pas.estDefiniParObjetDans(listeOb);
	}	     
 	
  @Override
  public double resultat(boolean infoRandom) throws ErreurCalculException {
  	double[] val = new double[1];
  	double dinddeb = indiceDebut.resultat(infoRandom);
  	if ((dinddeb > Integer.MAX_VALUE) || (dinddeb < Integer.MIN_VALUE) || (dinddeb != Math.floor(dinddeb))) throw new ErreurCalculException();
  	double dindfin = indiceFin.resultat(infoRandom);
  	if ((dindfin > Integer.MAX_VALUE) || (dindfin < Integer.MIN_VALUE) || (dindfin != Math.floor(dindfin))
  			|| (dindfin < dinddeb)) throw new ErreurCalculException();
  	double dpas = pas.resultat(infoRandom);
  	if ((dpas > Integer.MAX_VALUE) || (dpas < Integer.MIN_VALUE)
  			|| (dpas != Math.floor(dpas)) || (dpas <= 0)) throw new ErreurCalculException();
  	listeProprietaire.nombreIterations += (dindfin-dinddeb)/dpas;
  	if (listeProprietaire.nombreIterations > CCalculBase.nombreMaxiIterations) throw new ErreurCalculException(true);
  	int inddeb = (int) dinddeb;
  	int indfin = (int) dindfin;
  	int valpas = (int) dpas;
  	double somme = 0;
  	for (int i = inddeb; i <= indfin; i+= valpas) {
    	val[0] = i;
    	somme += calculASommer.resultatFonction(infoRandom, val);
  	}
  	return somme;
  }
  
  @Override
  public void resultatComplexe(boolean infoRandom, Complexe zRes)  throws ErreurCalculException {
  	Complexe[] val = new Complexe[1];
  	val[0] = new Complexe();
  	Complexe inddeb = new Complexe();
  	indiceDebut.resultatComplexe(infoRandom, inddeb);
  	if (inddeb.y != 0) throw new ErreurCalculException();
  	double dinddeb = inddeb.x;
  	if ((dinddeb > Integer.MAX_VALUE) || (dinddeb < Integer.MIN_VALUE) || (dinddeb != Math.floor(dinddeb))) throw new ErreurCalculException();
  	Complexe indfin = new Complexe();
  	indiceFin.resultatComplexe(infoRandom, indfin);
  	if (inddeb.y != 0) throw new ErreurCalculException();
  	double dindfin = indfin.x;
  	if ((dindfin > Integer.MAX_VALUE) || (dindfin < Integer.MIN_VALUE) || (dindfin != Math.floor(dindfin))
  			|| (dindfin < dinddeb)) throw new ErreurCalculException();
  	Complexe valpas = new Complexe();
  	pas.resultatComplexe(infoRandom, valpas);
  	if (valpas.y != 0) throw new ErreurCalculException();
  	double dvalpas = valpas.x;
  	if ((dvalpas > Integer.MAX_VALUE) || (dvalpas < Integer.MIN_VALUE)
  			|| (dvalpas != Math.floor(dvalpas)) || (dvalpas <= 0)) throw new ErreurCalculException();
  	listeProprietaire.nombreIterations += (dindfin-dinddeb)/dvalpas;
  	if (listeProprietaire.nombreIterations > CCalculBase.nombreMaxiIterations) throw new ErreurCalculException(true);
  	zRes.x = 0;
  	zRes.y = 0;
  	Complexe z = new Complexe(0, 0);
  	for (int i = (int)inddeb.x; i <= (int) indfin.x; i+= (int)valpas.x) {
    	val[0].x = i;
    	val[0].y = 0;
    	calculASommer.resultatFonctionComplexe(infoRandom, val, z);
    	zRes.x += z.x;
    	zRes.y += z.y;
  	}
  	return;  
  }

  public double resultatFonction(boolean infoRandom, double valeurParametre) throws ErreurCalculException {
  	double[] val = new double[2];
  	val[0] = valeurParametre;
  	double dinddeb = indiceDebut.resultatFonction(infoRandom, valeurParametre);
  	if ((dinddeb > Integer.MAX_VALUE) || (dinddeb < Integer.MIN_VALUE) || (dinddeb != Math.floor(dinddeb))) throw new ErreurCalculException();
  	double dindfin = indiceFin.resultatFonction(infoRandom, valeurParametre);
  	if ((dindfin > Integer.MAX_VALUE) || (dindfin < Integer.MIN_VALUE) || (dindfin != Math.floor(dindfin))
  			|| (dindfin < dinddeb)) throw new ErreurCalculException();
  	double dpas = pas.resultatFonction(infoRandom, valeurParametre);
  	if ((dpas > Integer.MAX_VALUE) || (dpas < Integer.MIN_VALUE)
  			|| (dpas != Math.floor(dpas)) || (dpas <= 0)) throw new ErreurCalculException();
  	listeProprietaire.nombreIterations += (dindfin-dinddeb)/dpas;
  	if (listeProprietaire.nombreIterations > CCalculBase.nombreMaxiIterations) throw new ErreurCalculException(true);
  	int inddeb = (int) dinddeb;
  	int indfin = (int) dindfin;
  	int valpas = (int) dpas;
  	double somme = 0;
  	for (int i = inddeb; i <= indfin; i+= valpas) {
    	val[1] = i;
    	somme += calculASommer.resultatFonction(infoRandom, val);
  	}
  	return somme;
  }
	
  // calculASommer pointe sur une fonction de n+1 variables, n tant la  dmiension de valeurParamre
  // La dernire variable sert  appeler la fonction pour les diffrentes valeurs de l'indice.
  @Override
  public double resultatFonction(boolean infoRandom, double[] valeurParametre) throws ErreurCalculException {
  	int n = valeurParametre.length;
  	double[] val = new double[n + 1];
  	for (int i = 0; i < n; i++) val[i] = valeurParametre[i];
  	double dinddeb = indiceDebut.resultatFonction(infoRandom, valeurParametre);
  	if ((dinddeb > Integer.MAX_VALUE) || (dinddeb < Integer.MIN_VALUE) || (dinddeb != Math.floor(dinddeb))) throw new ErreurCalculException();
  	double dindfin = indiceFin.resultatFonction(infoRandom, valeurParametre);
  	if ((dindfin > Integer.MAX_VALUE) || (dindfin < Integer.MIN_VALUE) || (dindfin != Math.floor(dindfin))
  			|| (dindfin < dinddeb)) throw new ErreurCalculException();
  	double dpas = pas.resultatFonction(infoRandom, valeurParametre);
  	if ((dpas > Integer.MAX_VALUE) || (dpas < Integer.MIN_VALUE)
  			|| (dpas != Math.floor(dpas)) || (dpas <= 0)) throw new ErreurCalculException();
  	listeProprietaire.nombreIterations += (dindfin-dinddeb)/dpas;
  	if (listeProprietaire.nombreIterations > CCalculBase.nombreMaxiIterations) throw new ErreurCalculException(true);
  	int inddeb = (int) dinddeb;
  	int indfin = (int) dindfin;
  	int valpas = (int) dpas;
  	double somme = 0;
  	for (int i = inddeb; i <= indfin; i+= valpas) {
    	val[n] = i;
    	somme += calculASommer.resultatFonction(infoRandom, val);
  	}
  	return somme;
  }
  
  void  resultatFonctionComplexe(boolean infoRandom, Complexe valeurParametre, Complexe zRes)
  throws ErreurCalculException
  {
  	Complexe[] val = new Complexe[2];
  	val[0] = valeurParametre;
  	val[1] = new Complexe();
  	Complexe inddeb = new Complexe();
  	indiceDebut.resultatFonctionComplexe(infoRandom, valeurParametre, inddeb);
  	if (inddeb.y != 0) throw new ErreurCalculException();
  	double dinddeb = inddeb.x;
  	if ((dinddeb > Integer.MAX_VALUE) || (dinddeb < Integer.MIN_VALUE) || (dinddeb != Math.floor(dinddeb))) throw new ErreurCalculException();
  	Complexe indfin = new Complexe();
  	indiceFin.resultatFonctionComplexe(infoRandom, valeurParametre, indfin);
  	if (indfin.y != 0) throw new ErreurCalculException();
  	double dindfin = indfin.x;
  	if ((dindfin > Integer.MAX_VALUE) || (dindfin < Integer.MIN_VALUE) || (dindfin != Math.floor(dindfin))
  			|| (dindfin < dinddeb)) throw new ErreurCalculException();
  	Complexe valpas = new Complexe();
  	pas.resultatFonctionComplexe(infoRandom, valeurParametre, valpas);
  	if (valpas.y != 0) throw new ErreurCalculException();
  	double dvalpas = valpas.x;
  	if ((dvalpas > Integer.MAX_VALUE) || (dvalpas < Integer.MIN_VALUE)
  			|| (dvalpas != Math.floor(dvalpas)) || (dvalpas <= 0)) throw new ErreurCalculException();
  	listeProprietaire.nombreIterations += (dindfin-dinddeb)/dvalpas;
  	if (listeProprietaire.nombreIterations > CCalculBase.nombreMaxiIterations) throw new ErreurCalculException(true);  	
  	zRes.x = 0;
  	zRes.y = 0;
  	Complexe z = new Complexe(0, 0);
  	for (int i = (int)inddeb.x; i <= (int) indfin.x; i+= (int)valpas.x) {
    	val[1].x = i;
    	val[1].y = 0;
    	calculASommer.resultatFonctionComplexe(infoRandom, val, z);
    	zRes.x += z.x;
    	zRes.y += z.y;
  	}
  	return;
  }
  
  void  resultatFonctionComplexe(boolean infoRandom, Complexe[] valeurParametre, Complexe zRes)
  throws ErreurCalculException
  {
  	int n = valeurParametre.length;
  	Complexe[] val = new Complexe[n + 1];
  	for (int i = 0; i < n; i++) val[i] = new Complexe(valeurParametre[i]);
  	val[n] = new Complexe();
  	Complexe inddeb = new Complexe();
  	indiceDebut.resultatFonctionComplexe(infoRandom, valeurParametre, inddeb);
  	if (inddeb.y != 0) throw new ErreurCalculException();
  	double dinddeb = inddeb.x;
  	if ((dinddeb > Integer.MAX_VALUE) || (dinddeb < Integer.MIN_VALUE) || (dinddeb != Math.floor(dinddeb))) throw new ErreurCalculException();
  	Complexe indfin = new Complexe();
  	indiceFin.resultatFonctionComplexe(infoRandom, valeurParametre, indfin);
  	if (indfin.y != 0) throw new ErreurCalculException();
  	double dindfin = indfin.x;
  	if ((dindfin > Integer.MAX_VALUE) || (dindfin < Integer.MIN_VALUE) || (dindfin != Math.floor(dindfin))
  			|| (dindfin < dinddeb)) throw new ErreurCalculException();
  	Complexe valpas = new Complexe();
  	pas.resultatFonctionComplexe(infoRandom, valeurParametre, valpas);
  	if (valpas.y != 0) throw new ErreurCalculException();
  	double dvalpas = valpas.x;
  	if ((dvalpas > Integer.MAX_VALUE) || (dvalpas < Integer.MIN_VALUE)
  			|| (dvalpas != Math.floor(dvalpas)) || (dvalpas <= 0)) throw new ErreurCalculException();
  	listeProprietaire.nombreIterations += (dindfin-dinddeb)/dvalpas;
  	if (listeProprietaire.nombreIterations > CCalculBase.nombreMaxiIterations) throw new ErreurCalculException(true);
  	zRes.x = 0;
  	zRes.y = 0;
  	Complexe z = new Complexe(0, 0);
  	for (int i = (int)inddeb.x; i <= (int) indfin.y; i+= (int)valpas.x) {
    	val[n].x = i;
    	val[n].y = 0;
    	calculASommer.resultatFonctionComplexe(infoRandom, val, z);
    	zRes.x += z.x;
    	zRes.y += z.y;
  	}
  	return; 
  }  
  
  @Override
	public String chaineCalcul(String[] varFor) {
  	String[] varFor2;
  	int n = 0;
  	if (varFor == null) varFor2 = new String[1];
  	else {
	  	n = varFor.length; 
	  	varFor2 = new String[n+1];
  	}
  	for (int i = 0; i < n; i++) varFor2[i] = varFor[i];
  	varFor2[n] = nomVariable;
  	return getString("somme") + "(" + calculASommer.chaineCalculSansPar(varFor2) + ","
  		+ nomVariable + "," + indiceDebut.chaineCalculSansPar(varFor) + ","
  		+ indiceFin.chaineCalculSansPar(varFor) + "," + pas.chaineCalculSansPar(varFor) + ")";
  }

  @Override
	public String chaineLatex(String[] varFor) {
  	String[] varFor2;
  	int n = 0;
  	if (varFor == null) varFor2 = new String[1];
  	else {
	  	n = varFor.length; 
	  	varFor2 = new String[n+1];
  	}
  	for (int i = 0; i < n; i++) varFor2[i] = varFor[i];
  	varFor2[n] = nomVariable;
  	// Modification version 4.7.7 : on renvoie un vrai symbole LaTeX de somme
  	String up = "";
  	if (pas.nature() == CCalculBase.natConstante) {
  		CConstante c = (CConstante) pas;
  		if (c.valeur == 1) up = indiceFin.chaineLatex(varFor);
  	}
  	if (up.length() == 0) up = indiceFin.chaineLatex(varFor) + "\\text{, " + getString("step") + " }"+pas.chaineLatex(varFor);
  	return "\\sum\\limits_{"+nomVariable + "=" + indiceDebut.chaineLatexSansPar(varFor) + "}" + "^{"+ up+"}"+ "{" + calculASommer.chaineLatex(varFor2) + "}";
  	/*
  	return getString("somme") + parouvLatex + calculASommer.chaineLatexSansPar(varFor2) + ","
  		+ nomVariable + "," + indiceDebut.chaineLatexSansPar(varFor) + ","
  		+ indiceFin.chaineLatexSansPar(varFor) + "," + pas.chaineLatexSansPar(varFor) + parferLatex;
  		*/
  }
  
  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    nomVariable = inps.readUTF();
    calculASommer = (CCalculBase) (inps.readObject(list));
		indiceDebut = (CCalculBase) (inps.readObject(list));
		indiceFin = (CCalculBase) (inps.readObject(list));
		pas = (CCalculBase) (inps.readObject(list));
		
  }  

  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    oups.writeUTF(nomVariable);
    oups.writeObject(calculASommer);
    oups.writeObject(indiceDebut);
    oups.writeObject(indiceFin);
    oups.writeObject(pas);
  }
  
  @Override
  public boolean deriveePossible(int indiceVariable) {
  	return calculASommer.deriveePossible(indiceVariable)
  		&& !indiceDebut.dependDeVariable(indiceVariable) && !indiceFin.dependDeVariable(indiceVariable)
  		&& !pas.dependDeVariable(indiceVariable);
  }
  
  // Ajout version 4.6
  @Override
  public CCalculBase calculAvecValeursRemplacees() {
  	return new CSommeDansFormule(listeProprietaire, calculASommer.calculAvecValeursRemplacees(),
  			indiceDebut.calculAvecValeursRemplacees(), indiceFin.calculAvecValeursRemplacees(),
  			pas.calculAvecValeursRemplacees(), nomVariable) ;
  }
  
}
